@@ -1,5 +1,5 @@
 #include <stdio.h>
-#include <limits>
+#include <cfloat>
 #include <string>
 #include <set>
 #include <vector>
@@ -49,8 +49,8 @@ bool RPCMetricsFilter::client_end(SubTask *task, RPCModuleData& data)
 {
 	this->gauge(METRICS_REQUEST_COUNT)->increase();
 	this->counter(METRICS_REQUEST_METHOD)->add(
-				{{"service", data[SRPC_SERVICE_NAME]},
-				 {"method",  data[SRPC_METHOD_NAME] }})->increase();
+				{{"service", data[OTLP_SERVICE_NAME]},
+				 {"method",  data[OTLP_METHOD_NAME] }})->increase();
 	this->summary(METRICS_REQUEST_LATENCY)->observe(atoll(data[SRPC_DURATION].data()));
 
 	return true;
@@ -60,8 +60,8 @@ bool RPCMetricsFilter::server_end(SubTask *task, RPCModuleData& data)
 {
 	this->gauge(METRICS_REQUEST_COUNT)->increase();
 	this->counter(METRICS_REQUEST_METHOD)->add(
-				{{"service", data[SRPC_SERVICE_NAME]},
-				 {"method",  data[SRPC_METHOD_NAME] }})->increase();
+				{{"service", data[OTLP_SERVICE_NAME]},
+				 {"method",  data[OTLP_METHOD_NAME] }})->increase();
 	this->summary(METRICS_REQUEST_LATENCY)->observe(atoll(data[SRPC_DURATION].data()));
 
 	return true;
@@ -311,7 +311,7 @@ void RPCMetricsPull::Collector::collect_histogram_each(RPCVar *histogram,
 													   size_t current_count)
 {
 	this->report_output += histogram->get_name();
-	if (bucket_boundary != std::numeric_limits<double>::max())
+	if (bucket_boundary != DBL_MAX)
 	{
 		this->report_output += "_bucket{le=\"" +
 							   std::to_string(bucket_boundary) + "\"} ";
@@ -428,20 +428,36 @@ SubTask *RPCMetricsOTel::create(RPCModuleData& data)
 	ResourceMetrics *rm = req.add_resource_metrics();
 	Resource *resource = rm->mutable_resource();
 	InstrumentationLibraryMetrics *metrics = rm->add_instrumentation_library_metrics();
+	KeyValue *attribute;
+	AnyValue *value;
+
+	auto iter = data.find(OTLP_METHOD_NAME);
+	if (iter != data.end())
+	{
+		attribute = resource->add_attributes();
+		attribute->set_key(OTLP_METHOD_NAME);
+		value = attribute->mutable_value();
+		value->set_string_value(iter->second);
+	}
 
 	this->mutex.lock();
-	for (const auto& attr : attributes)
+	for (const auto& attr : this->attributes)
 	{
 		KeyValue *attribute = resource->add_attributes();
 		attribute->set_key(attr.first);
 		AnyValue *value = attribute->mutable_value();
 		value->set_string_value(attr.second);
 	}	
+	this->mutex.unlock();
 
-	KeyValue *attribute = resource->add_attributes();
-	attribute->set_key(OTLP_SERVICE_NAME_KEY);
-	AnyValue *value = attribute->mutable_value();
-	value->set_string_value(data[SRPC_SERVICE_NAME]);
+	iter = data.find(OTLP_SERVICE_NAME); // if attributes also set service.name, data takes precedence
+	if (iter != data.end())
+	{
+		attribute = resource->add_attributes();
+		attribute->set_key(OTLP_SERVICE_NAME);
+		value = attribute->mutable_value();
+		value->set_string_value(iter->second);
+	}
 
 	this->expose(metrics);
 
@@ -592,7 +608,7 @@ void RPCMetricsOTel::Collector::collect_histogram_each(RPCVar *histogram,
 	HistogramDataPoint *data_points = static_cast<HistogramDataPoint *>(this->current_msg);
 	data_points->add_bucket_counts(current_count);
 
-	if (bucket_boundary != std::numeric_limits<double>::max())
+	if (bucket_boundary != DBL_MAX)
 		data_points->add_explicit_bounds(bucket_boundary);
 }
 
